#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-11-09
#
# Monitor Check Point Maestro SMO SGM Counters
#
# 2021-09-10: rewrtitten for CMK 2.0
# 2021-09-11: added metrics
#
# ToDo: add WATO options
#
# sample snmpwalk (for one existing SGM)
# .1.3.6.1.4.1.2620.1.48.20.25.1.1.1.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.48.20.25.1.2.1.0 = STRING: "1_01"
# .1.3.6.1.4.1.2620.1.48.20.25.1.3.1.0 = STRING: "72609"
# .1.3.6.1.4.1.2620.1.48.20.25.1.4.1.0 = STRING: "0"
# .1.3.6.1.4.1.2620.1.48.20.25.1.5.1.0 = STRING: "76"
# .1.3.6.1.4.1.2620.1.48.20.25.1.6.1.0 = STRING: "37"
# .1.3.6.1.4.1.2620.1.48.20.25.1.7.1.0 = STRING: "0"
# .1.3.6.1.4.1.2620.1.48.20.25.1.8.1.0 = STRING: "0"
# .1.3.6.1.4.1.2620.1.48.20.25.1.9.1.0 = STRING: "0"
# .1.3.6.1.4.1.2620.1.48.20.25.1.10.1.0 = STRING: "2"
# .1.3.6.1.4.1.2620.1.48.20.25.1.11.1.0 = STRING: "0"
# .1.3.6.1.4.1.2620.1.48.20.25.1.12.1.0 = STRING: "9"
#
# # sample snmpwalk (for one none existing SGM)
# .1.3.6.1.4.1.2620.1.48.20.25.1.1.3.0 = Gauge32: 3
# .1.3.6.1.4.1.2620.1.48.20.25.1.2.3.0 = STRING: "1_03"
# .1.3.6.1.4.1.2620.1.48.20.25.1.3.3.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.48.20.25.1.4.3.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.48.20.25.1.5.3.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.48.20.25.1.6.3.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.48.20.25.1.7.3.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.48.20.25.1.8.3.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.48.20.25.1.9.3.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.48.20.25.1.10.3.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.48.20.25.1.11.3.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.48.20.25.1.12.3.0 = STRING: "N/A"
#
#
# sample info
# [
#  [u'1_01', u'72609', u'0', u'76', u'37', u'0', u'0', u'0', u'2', u'0', u'9'],
#  [u'1_02', u'39193', u'0', u'8', u'29', u'0', u'0', u'1', u'1', u'0', u'7'],
#  [u'1_03', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'1_04', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'1_05', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'1_06', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'1_07', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'1_08', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'1_09', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'1_10', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'1_11', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'1_12', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'1_13', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'1_14', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'2_01', u'0', u'0', u'0', u'25', u'0', u'0', u'0', u'1', u'0', u'5'],
#  [u'2_02', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'2_03', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'2_04', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'2_05', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'2_06', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'2_07', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'2_08', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'2_09', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'2_10', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'2_11', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'2_12', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'2_13', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A'],
#  [u'2_14', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A', u'N/A']
# ]
#
import time
from dataclasses import dataclass
from typing import List, Dict, NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    Metric,
    State,
    SNMPTree,
    all_of,
    startswith,
    any_of,
    equals,
    check_levels,
    render,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)


@dataclass
class CheckpointAsgCounter:
    label:str
    metric: str
    unit: str
    value: float
    notice: bool


def parse_checkpoint_asg_counters(string_table: StringTable) -> Dict[str, List[CheckpointAsgCounter]]:
    items = {}
    for sgm in string_table:
        asgBladeId, asgThroughput, asgConnRate, asgPacketRate, asgConcurrConnNum, asgAccelLoadAverage, \
        asgAccelLoadMin, asgAccelLoadMax, asgInstanceLoadAverage, asgInstanceLoadMin, asgInstanceLoadMax = sgm

        if not asgThroughput == asgConnRate == 'N/A':  # do not add blades without data
            items[asgBladeId] = [
                CheckpointAsgCounter(metric='throughput', value=int(asgThroughput), label='Throughput', unit=' Bytes/s', notice=False),
                CheckpointAsgCounter(metric='packet_rate', value= int(asgPacketRate), label='Packet rate', unit='/s', notice=False),
                CheckpointAsgCounter(metric='conn_rate', value= int(asgConnRate), label='Connection rate', unit='/s', notice=False),
                CheckpointAsgCounter(metric='concurr_conn_num', value=int(asgConcurrConnNum), label='Connections', unit=' connections', notice=False),
                CheckpointAsgCounter(metric='accel_load_average', value=int(asgAccelLoadAverage), label='Accel load', unit='%', notice=True),
                CheckpointAsgCounter(metric='instance_load_average', value=int(asgInstanceLoadAverage), label='Instance load', unit='%', notice=True),
                CheckpointAsgCounter(metric='accel_load_min', value=int(asgAccelLoadMin), label='Accel load min', unit='%', notice=True),
                CheckpointAsgCounter(metric='accel_load_max', value=int(asgAccelLoadMax), label='Accel load max', unit='%', notice=True),
                CheckpointAsgCounter(metric='instance_load_min', value=int(asgInstanceLoadMin), label='Instance load min', unit='%', notice=True),
                CheckpointAsgCounter(metric='instance_load_max', value=int(asgInstanceLoadMax), label='Instance load mox', unit='%', notice=True),
            ]

    return items


def discovery_checkpoint_asg_counters(section: Dict[str, List[CheckpointAsgCounter]]) -> DiscoveryResult:
    for item in section.keys():
        yield Service(item=item)


def check_checkpoint_asg_counters(item, params, section: Dict[str, List[CheckpointAsgCounter]]) -> CheckResult:
    try:
        counters = section[item]
    except KeyError:
        yield Result(state=State.UNKNOWN, notice='Item not found in SNMP data')
        return

    for counter in counters:
        yield from check_levels(
            value=counter.value,
            label=counter.label,
            metric_name=f'checkpoint_asg_counters_{counter.metric}',
            render_func=lambda v: f'{v:.0f}{counter.unit}',
            notice_only=counter.notice,
        )


register.snmp_section(
    name='checkpoint_asg_counters',
    parse_function=parse_checkpoint_asg_counters,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.1.48.20.25.1',  # CHECKPOINT-MIB::asgCountersEntry
        oids=[
            '2',  # asgCountersBladeId
            '3',  # asgCountersThroughput
            '4',  # asgCountersConnRate
            '5',  # asgCountersPacketRate
            '6',  # asgCountersConcurrConnNum
            '7',  # asgCountersAccelLoadAverage
            '8',  # asgCountersAccelLoadMin
            '9',  # asgCountersAccelLoadMax
            '10',  # asgCountersInstanceLoadAverage
            '11',  # asgCountersInstanceLoadMin
            '12',  # asgCountersInstanceLoadMax
        ]
    ),
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_asg_counters',
    service_name='ASG SGM counters %s',
    discovery_function=discovery_checkpoint_asg_counters,
    check_function=check_checkpoint_asg_counters,
    check_ruleset_name='checkpoint_asg_counters',
    check_default_parameters={},
)
