#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#

from cmk.gui.i18n import _

from cmk.gui.plugins.metrics import (
    metric_info,
    graph_info,
    perfometer_info,
)

metric_info['checkpoint_asg_counters_throughput'] = {
    'title': _('Throughput'),
    'unit': 'bytes/s',
    'color': '26/a',
}

metric_info['checkpoint_asg_counters_packet_rate'] = {
    'title': _('Packet Rate'),
    'unit': '1/s',
    'color': '32/a',
}

metric_info['checkpoint_asg_counters_conn_rate'] = {
    'title': _('Connection Rate'),
    'unit': '1/s',
    'color': '23/a',
}

metric_info['checkpoint_asg_counters_concurr_conn_num'] = {
    'title': _('Connections'),
    'unit': 'count',
    'color': '14/a',
}

metric_info['checkpoint_asg_counters_accel_load_average'] = {
    'title': _('Accellerator load (ave)'),
    'unit': '%',
    'color': '11/a',
}
metric_info['checkpoint_asg_counters_accel_load_min'] = {
    'title': _('Accellerator load (min)'),
    'unit': '%',
    'color': '21/a',
}
metric_info['checkpoint_asg_counters_accel_load_max'] = {
    'title': _('Accellerator load (max)'),
    'unit': '%',
    'color': '31/a',
}

metric_info['checkpoint_asg_counters_instance_load_average'] = {
    'title': _('Instance load (ave)'),
    'unit': '%',
    'color': '22/a',
}
metric_info['checkpoint_asg_counters_instance_load_min'] = {
    'title': _('Instance load (min)'),
    'unit': '%',
    'color': '32/a',
}
metric_info['checkpoint_asg_counters_instance_load_max'] = {
    'title': _('Instance load (max)'),
    'unit': '%',
    'color': '42/a',
}

graph_info['checkpoint_asg_counters_throughput'] = {
    'title': _('Check Point SGM Counter Throughput'),
    'metrics': [
        ('checkpoint_asg_counters_throughput', 'area'),
    ],
    'range': (0, 'checkpoint_asg_counters_throughput:max'),
}

graph_info['checkpoint_asg_counters_packet_rate'] = {
    'title': _('Check Point SGM Counter Packet Rate'),
    'metrics': [
        ('checkpoint_asg_counters_packet_rate', 'area'),
    ],
    'range': (0, 'checkpoint_asg_counters_packet_rate:max'),
}

graph_info['checkpoint_asg_counters_conn_rate'] = {
    'title': _('Check Point SGM Counter Connection Rate'),
    'metrics': [
        ('checkpoint_asg_counters_conn_rate', 'area'),
    ],
    'range': (0, 'checkpoint_asg_counters_conn_rate:max'),
}

graph_info['checkpoint_asg_counters_concurr_conn_num'] = {
    'title': _('Check Point SGM Counter Current Connections'),
    'metrics': [
        ('checkpoint_asg_counters_concurr_conn_num', 'area'),
    ],
    'range': (0, 'checkpoint_asg_counters_concurr_conn_num:max'),
}

graph_info['checkpoint_asg_counters_accel_load'] = {
    'title': _('Check Point SGM Counter Accell Load'),
    'metrics': [
        ('checkpoint_asg_counters_accel_load_min', 'line'),
        ('checkpoint_asg_counters_accel_load_average', 'line'),
        ('checkpoint_asg_counters_accel_load_max', 'line'),
    ],
    'range': (0, 110),
    'optional_metrics': [
        'checkpoint_asg_counters_accel_load_min',
        'checkpoint_asg_counters_accel_load_max',
    ],
}

graph_info['checkpoint_asg_counters_instance_load'] = {
    'title': _('Check Point SGM Counter Instance Load'),
    'metrics': [
        ('checkpoint_asg_counters_instance_load_min', 'line'),
        ('checkpoint_asg_counters_instance_load_average', 'line'),
        ('checkpoint_asg_counters_instance_load_max', 'line'),
    ],
    'range': (0, 110),
    'optional_metrics': [
        'checkpoint_asg_counters_instance_load_min',
        'checkpoint_asg_counters_instance_load_max',
    ],
}

perfometer_info.append(('stacked', [
    {
        'type': 'logarithmic',
        'metric': 'checkpoint_asg_counters_throughput',
        'half_value': 2592000.0,
        'exponent': 2,
    },
    {
        'type': 'logarithmic',
        'metric': 'checkpoint_asg_counters_concurr_conn_num',
        'half_value': 100000.0,
        'exponent': 2,
    },
]))
